@@ -3,6 +3,7 @@
 /* 
  * Copyright (C) 2001 Havoc Pennington (some code in here from
  * libgnomeui, (C) Tom Tromey, Carsten Schaar)
+ * Copyright (C) 2004, 2005 Elijah Newren
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,6 +24,7 @@
 #include <config.h>
 
 #include "session.h"
+#include <X11/Xatom.h>
 
 #include <time.h>
 
@@ -726,47 +728,6 @@ window_type_from_string (const char *str)
     return META_WINDOW_NORMAL;
 }
 
-static const char*
-window_gravity_to_string (int gravity)
-{
-  switch (gravity)
-    {
-    case NorthWestGravity:
-      return "NorthWestGravity";
-      break;
-    case NorthGravity:
-      return "NorthGravity";
-      break;
-    case NorthEastGravity:
-      return "NorthEastGravity";
-      break;
-    case WestGravity:
-      return "WestGravity";
-      break;
-    case CenterGravity:
-      return "CenterGravity";
-      break;
-    case EastGravity:
-      return "EastGravity";
-      break;
-    case SouthWestGravity:
-      return "SouthWestGravity";
-      break;
-    case SouthGravity:
-      return "SouthGravity";
-      break;
-    case SouthEastGravity:
-      return "SouthEastGravity";
-      break;
-    case StaticGravity:
-      return "StaticGravity";
-      break;
-    default:
-      return "NorthWestGravity";
-      break;
-    }
-}
-  
 static int
 window_gravity_from_string (const char *str)
 {
@@ -993,7 +954,7 @@ save_state (void)
                 fputs ("    <minimized/>\n", outfile);
 
               /* Maximized */
-              if (window->maximized)
+              if (META_WINDOW_MAXIMIZED (window))
 		{
                   fprintf (outfile,
                            "    <maximized saved_x=\"%d\" saved_y=\"%d\" saved_width=\"%d\" saved_height=\"%d\"/>\n", 
@@ -1005,17 +966,10 @@ save_state (void)
               
               /* Workspaces we're on */
               {
-                GList *w;
-                w = window->workspaces;
-                while (w != NULL)
-                  {
-                    int n;
-                    n = meta_workspace_index (w->data);
-                    fprintf (outfile,
-                             "    <workspace index=\"%d\"/>\n", n);
-
-                    w = w->next;
-                  }
+                int n;
+                n = meta_workspace_index (window->workspace);
+                fprintf (outfile,
+                         "    <workspace index=\"%d\"/>\n", n);
               }
 
               /* Gravity */
@@ -1026,7 +980,7 @@ save_state (void)
                 fprintf (outfile,
                          "    <geometry x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" gravity=\"%s\"/>\n",
                          x, y, w, h,
-                         window_gravity_to_string (window->size_hints.win_gravity));
+                         meta_gravity_to_string (window->size_hints.win_gravity));
               }
               
               fputs ("  </window>\n", outfile);
@@ -1485,7 +1439,7 @@ start_element_handler  (GMarkupParseContext *context,
                   pd->info->rect.y,
                   pd->info->rect.width,
                   pd->info->rect.height,
-                  window_gravity_to_string (pd->info->gravity));
+                  meta_gravity_to_string (pd->info->gravity));
     }
   else
     {
@@ -1851,6 +1805,8 @@ warn_about_lame_clients_and_finish_interact (gboolean shutdown)
   GError *err;
   GIOChannel *channel;
   LameClientsDialogData *d;
+  Time timestamp;
+  char timestampbuf[32];
   
   lame = NULL;
   displays = meta_displays_list ();
@@ -1893,8 +1849,32 @@ warn_about_lame_clients_and_finish_interact (gboolean shutdown)
   
   lame = g_slist_sort (lame, (GCompareFunc) windows_cmp_by_title);
 
+  
+    {
+      XEvent property_event;
+      MetaDisplay *display;
+
+      display = meta_displays_list ()->data;
+
+      /* Using the property XA_PRIMARY because it's safe; nothing
+       * would use it as a property. The type doesn't matter.
+       */
+      XChangeProperty (display->xdisplay,
+                       display->leader_window,
+                       XA_PRIMARY, XA_STRING, 8,
+                       PropModeAppend, NULL, 0);
+      XWindowEvent (display->xdisplay,
+                    display->leader_window,
+                    PropertyChangeMask,
+                    &property_event);
+
+      timestamp = property_event.xproperty.time;
+    }
+  sprintf (timestampbuf, "%lu", timestamp);
+
   len = g_slist_length (lame);
   len *= 2; /* titles and also classes */
+  len += 2; /* --timestamp flag and actual timestamp */
   len += 1; /* NULL term */
   len += 2; /* metacity-dialog command and option */
   
@@ -1904,6 +1884,10 @@ warn_about_lame_clients_and_finish_interact (gboolean shutdown)
 
   argv[i] = METACITY_LIBEXECDIR"/metacity-dialog";
   ++i;
+  argv[i] = "--timestamp";
+  ++i;
+  argv[i] = timestampbuf;
+  ++i;
   argv[i] = "--warn-about-no-sm-support";
   ++i;
   
